/*
 *  Copyright (c) 2001-2003, Jean Tessier
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  	* Redistributions of source code must retain the above copyright
 *  	  notice, this list of conditions and the following disclaimer.
 *  
 *  	* Redistributions in binary form must reproduce the above copyright
 *  	  notice, this list of conditions and the following disclaimer in the
 *  	  documentation and/or other materials provided with the distribution.
 *  
 *  	* Neither the name of Jean Tessier nor the names of his contributors
 *  	  may be used to endorse or promote products derived from this software
 *  	  without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jeantessier.diff;

import org.apache.log4j.*;

import com.jeantessier.classreader.*;

/**
 *  Documents the difference, if any, for a given programming
 *  element that can be deprecated through the use of javadoc
 *  tags.
 */
public class DocumentableDifferences extends DecoratorDifferences {
	private boolean new_documentation;
	private boolean removed_documentation;

	/**
	 *  Only the DifferencesFactory can create instances of this class.
	 */
	DocumentableDifferences(Differences component, Validator old_validator, Validator new_validator) {
		super(component);

		Logger.getLogger(getClass()).debug("Begin " + Name());

		NewDocumentation(!old_validator.IsAllowed(component.Name()) && new_validator.IsAllowed(component.Name()));
		RemovedDocumentation(old_validator.IsAllowed(component.Name()) && !new_validator.IsAllowed(component.Name()));

		Logger.getLogger(getClass()).debug("End   " + Name() + ": " + (IsEmpty() ? "empty" : "not empty"));
	}

	public boolean NewDocumentation() {
		Logger.getLogger(getClass()).debug(Name() + " NewDocumentation(): " + new_documentation);
		return new_documentation;
	}

	public void NewDocumentation(boolean new_documentation) {
		this.new_documentation = new_documentation;
	}

	public boolean RemovedDocumentation() {
		Logger.getLogger(getClass()).debug(Name() + " RemovedDocumentation(): " + removed_documentation);
		return removed_documentation;
	}

	public void RemovedDocumentation(boolean removed_documentation) {
		this.removed_documentation = removed_documentation;
	}

	public boolean IsEmpty() {
		return
			!NewDocumentation() &&
			!RemovedDocumentation() &&
			Component().IsEmpty();
	}

	public void Accept(Visitor visitor) {
		visitor.VisitDocumentableDifferences(this);
	}
}
